@@ -2,7 +2,7 @@
 # This package is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 #
-# $Id: ModInt.pm 51 2015-04-29 22:09:44Z demetri $
+# $Id: ModInt.pm 57 2015-05-17 22:39:05Z demetri $
 
 package Math::ModInt;
 
@@ -40,9 +40,9 @@ use overload (
 BEGIN {
     require Exporter;
     our @ISA       = qw(Exporter);
-    our @EXPORT_OK = qw(mod);
+    our @EXPORT_OK = qw(mod divmod qmod);
     our @CARP_NOT  = qw(Math::ModInt::ChineseRemainder);
-    our $VERSION   = '0.008';
+    our $VERSION   = '0.009';
 }
 
 sub _max_modulus_perl {
@@ -95,6 +95,14 @@ sub _best_class {
     UsageError->raise('positive integer modulus expected');
 }
 
+sub _load {
+    my ($class) = @_;
+    local $@;
+    ($loaded{$class} ||= eval "require $class")
+        or LoadingFailure->raise($class);
+    return $class;
+}
+
 sub _oadd {
     my ($this, $that) = @_;
     if (!ref $that || !$that->isa(__PACKAGE__)) {
@@ -170,22 +178,56 @@ sub _oneq {
         $this->modulus != $that->modulus;
 }
 
+# ----- fallback for implementation interface method -----
+
+sub _NEW2 {
+    my ($this, $int, $modulus) = @_;
+    my $that;
+    if (ref $this) {
+        $that = $this->_NEW($int);
+        $modulus = $this->modulus;
+    }
+    else {
+        $that = $this->_NEW($int, $modulus);
+    }
+    my $quot = ($int - $that->residue) / $modulus;
+    return ($quot, $that);
+}
+
 # ----- public methods -----
 
 # constructors
 
 sub mod {
-    my ($residue, $modulus) = @_;
-    my $class = _best_class($modulus);
-    ($loaded{$class} ||= eval "require $class")
-        or LoadingFailure->raise($class);
-    return $class->_NEW($residue, $modulus);
+    my ($int, $modulus) = @_;
+    my $class = _load(_best_class($modulus));
+    return $class->_NEW($int, $modulus);
+}
+
+sub divmod {
+    my ($int, $modulus) = @_;
+    my $class = _load(_best_class($modulus));
+    return $class->_NEW2($int, $modulus);
+}
+
+sub qmod {
+    my ($rat, $modulus) = @_;
+    my $class = _load(_best_class($modulus));
+    my $num   = $class->_NEW($rat->numerator, $modulus);
+    my $den   = $num->_NEW($rat->denominator);
+    return $num / $den;
 }
 
 sub new {
-    my ($this, $residue, $modulus) = @_;
-    return $this->_NEW($residue) if ref $this;
-    return mod($residue, $modulus);
+    my ($this, $int, $modulus) = @_;
+    return $this->_NEW($int) if ref $this;
+    return mod($int, $modulus);
+}
+
+sub new2 {
+    my ($this, $int, $modulus) = @_;
+    return $this->_NEW2($int) if ref $this;
+    return divmod($int, $modulus);
 }
 
 sub undefined {
@@ -276,22 +318,28 @@ Math::ModInt - modular integer arithmetic
 
 =head1 VERSION
 
-This documentation refers to version 0.008 of Math::ModInt.
+This documentation refers to version 0.009 of Math::ModInt.
 
 =head1 SYNOPSIS
 
-  use Math::ModInt qw(mod);
+  use Math::ModInt qw(mod divmod qmod);
 
   $a = mod(32, 127);            #  32 (mod 127)
   $b = $a->new(99);             #  99 (mod 127)
   $c = $a + $b;                 #   4 (mod 127)
   $d = $a**2 - $b/$a;           # 120 (mod 127)
+  ($i, $e) = divmod(32, 5);     #  6, mod(2, 5)
 
   $m = $d->modulus;             # 127
   $r = $d->residue;             # 120
   $s = $d->signed_residue;      #  -7
   $t = "$a";                    # 'mod(32, 127)'
 
+  use Math::BigRat;
+
+  $q = Math::BigRat->new('2/3');
+  $f = qmod($q, 5);             #   4 (mod 5)
+
 =head1 DESCRIPTION
 
 Math::ModInt provides overloaded operators for modular integer
@@ -318,21 +366,48 @@ is called.
 
 =item I<new>
 
-Called as a class method, C<Math::ModInt-E<gt>new($residue, $modulus)>
+Called as a class method, C<Math::ModInt-E<gt>new($int, $modulus)>
 creates a new object of a subclass appropriate for the given modulus
 and current platform.  The modulus must be a positive integer value.
 
-Called as an object method, C<$x-E<gt>new($residue)> creates a new
+Called as an object method, C<$x-E<gt>new($int)> creates a new
 object sharing both its type and modulus with the invocant object
 C<$x>.
 
+=item I<new2>
+
+The constructor method C<new2> is called exactly like C<new> but
+it returns two values: an integer division result and a modular
+integer object.
+
 =item I<mod>
 
 For convenience, C<mod> can be imported as an abbreviation for the
-class method constructor; C<mod($residue, $modulus)> is equivalent
-to C<Math::ModInt-E<gt>new($residue, $modulus)>.  Note that C<mod>
+class method constructor; C<mod($int, $modulus)> is equivalent
+to C<Math::ModInt-E<gt>new($int, $modulus)>.  Note that C<mod>
 has to be called as a plain function, not like a method.
 
+=item I<divmod>
+
+The C<divmod> function can be imported, too.  It takes two integers
+like C<mod> but it returns two values: an integer division result
+and a modular integer object.  Thus C<divmod($int, $modulus)> is
+equivalent to C<Math::ModInt-E<gt>new2($int, $modulus)>.
+
+=item I<qmod>
+
+Another importable constructor function is C<qmod>.  It can convert
+rational numbers to modular integers of a given modulus if the
+denominator is coprime to that modulus.
+The expression C<qmod($rat, $modulus)> is equivalent to
+C<mod($rat-E<gt>numerator, $modulus) / mod($rat-E<gt>denominator, $modulus)>.
+Note that C<$rat> must be an object with C<numerator> and C<denominator>
+methods, like e.g. an instance of Math::BigRat.
+
+Note also that mixed expressions with modular integers and rational
+numbers are not permitted.  Always convert rational numbers explicitly
+with C<qmod>.
+
 =item I<undefined>
 
 This method returns the C<undefined> placeholder object representing
@@ -556,6 +631,14 @@ that the exponent was negative while C<$x> had no modular inverse.
 
 =over 4
 
+=item I<_NEW2>
+
+This constructor method will be called like C<_NEW> but it must
+return two values: an integer division result and the newly created
+modular integer object.  It should be implemented if the underlying
+library offers an efficient way to calculate a quotient and remainder
+simultaneously.
+
 =item I<optimize_time>
 
 =item I<optimize_space>
