@@ -2,7 +2,7 @@
 # This package is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 #
-# $Id: BigInt.pm 43 2015-04-28 09:48:34Z demetri $
+# $Id: BigInt.pm 53 2015-05-17 21:36:50Z demetri $
 
 package Math::ModInt::BigInt;
 
@@ -11,6 +11,7 @@ use strict;
 use warnings;
 use Carp qw(croak);
 use Math::BigInt;
+use constant _ISSUE_CPAN_RT79060 => -1 != Math::BigInt->new(-2) / 3;
 
 # ----- object definition -----
 
@@ -26,7 +27,7 @@ use constant NFIELDS   => 2;
 BEGIN {
     require Math::ModInt;
     our @ISA     = 'Math::ModInt';
-    our $VERSION = '0.008';
+    our $VERSION = '0.009';
 }
 
 # ----- overridden methods -----
@@ -79,7 +80,7 @@ sub _INV {
 }
 
 sub _NEW {
-    my ($this, $residue, $modulus) = @_;
+    my ($this, $int, $modulus) = @_;
     my $class = ref $this;
     if ($class) {
         $modulus = $this->modulus;
@@ -87,12 +88,37 @@ sub _NEW {
     else {
         $class = $this;
     }
-    foreach my $arg ($residue, $modulus) {
+    foreach my $arg ($int, $modulus) {
         if (!ref($arg) || !$arg->isa('Math::BigInt')) {
             $arg = Math::BigInt->new($arg);
         }
     }
-    return bless [$residue % $modulus, $modulus], $class;
+    return bless [$int % $modulus, $modulus], $class;
+}
+
+sub _NEW2 {
+    my ($this, $int, $modulus) = @_;
+    my $class = ref $this;
+    if ($class) {
+        $modulus = $this->modulus;
+    }
+    else {
+        $class = $this;
+    }
+    foreach my $arg ($int, $modulus) {
+        if (!ref($arg) || !$arg->isa('Math::BigInt')) {
+            $arg = Math::BigInt->new($arg);
+        }
+    }
+    my ($quotient, $remainder);
+    if (_ISSUE_CPAN_RT79060 && $int->is_neg) {
+        $remainder = $int % $modulus;
+        $quotient = ($int - $remainder) / $modulus;
+    }
+    else {
+        ($quotient, $remainder) = $int->copy->bdiv($modulus);
+    }
+    return ($quotient, bless [$remainder, $modulus], $class);
 }
 
 sub residue {
@@ -115,7 +141,7 @@ Math::ModInt::BigInt - modular integer arithmetic, powered by Math::BigInt
 
 =head1 VERSION
 
-This documentation refers to version 0.008 of Math::ModInt::BigInt.
+This documentation refers to version 0.009 of Math::ModInt::BigInt.
 
 =head1 SYNOPSIS
 
