@@ -2,7 +2,7 @@
 # This package is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 #
-# $Id: GF2.pm 43 2015-04-28 09:48:34Z demetri $
+# $Id: GF2.pm 53 2015-05-17 21:36:50Z demetri $
 
 package Math::ModInt::GF2;
 
@@ -23,7 +23,7 @@ use constant NFIELDS   => 1;
 BEGIN {
     require Math::ModInt;
     our @ISA     = qw(Math::ModInt);
-    our $VERSION = '0.008';
+    our $VERSION = '0.009';
 }
 
 my @base = map { bless [$_] } 0..1;     # singletons
@@ -65,8 +65,18 @@ sub _INV {
 }
 
 sub _NEW {
-    my ($this, $residue) = @_;
-    return $base[$residue & 1];
+    my ($this, $int) = @_;
+    return $base[$int & 1];
+}
+
+sub _NEW2 {
+    my ($this, $int) = @_;
+    if ($int < 0) {
+        # no arithmetic shift operation for negative perl integers, alas
+        my $residue = $int & 1;
+        return (($int - $residue) / 2, $base[$residue]);
+    }
+    return ($int >> 1, $base[$int & 1]);
 }
 
 sub residue {
@@ -93,7 +103,7 @@ Math::ModInt::GF2 - integer arithmetic modulo 2
 
 =head1 VERSION
 
-This documentation refers to version 0.008 of Math::ModInt::GF2.
+This documentation refers to version 0.009 of Math::ModInt::GF2.
 
 =head1 SYNOPSIS
 
